// This is the main DLL file.

#include "stdafx.h"
#include <windows.h>

#include <stdio.h>

#include "DbMon.h"

void DBWinErrorMessage(char *message) {
	MessageBox(NULL, message, "DB_WIN_MESSAGE", 0);
}

/*
 *  This is a trimmed down version of a routine from DBWIN32 which credits dbmon...
 *
 *  "The main gist of this routine comes from the MSDN DBMON example."
 */
void DbMonClass::DbMon::EventThreadRoutine() {
    HANDLE hAckEvent = INVALID_HANDLE_VALUE;
    HANDLE SharedFile = INVALID_HANDLE_VALUE;
    LPVOID SharedMem = NULL;
    LPSTR  lpszString;
    DWORD  ret;
    DWORD  LastPid;
    LPDWORD pThisPid;
    SECURITY_ATTRIBUTES sa;
    SECURITY_DESCRIPTOR sd;
    const DWORD EXCEPT_LEAVE_EVENT_THREAD = 256;

    static  HANDLE     g_hReadyEvent = INVALID_HANDLE_VALUE;

	__try
    {

        /*
         *  Set the thread to a high priority because the debug strings
         *  being sent by the debugged app can make it wait if
         *  we don't grab the events quickly enough.
         */
        if ( !SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL) )
        {
            DBWinErrorMessage("Set thread priority failure in EventThreadRoutine()");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        /*
            Create a NULL DACL so that messages can be seen from things like
            ISAPI filters that run in the local system account.
        */
        sa.nLength = sizeof(SECURITY_ATTRIBUTES);
        sa.bInheritHandle = TRUE;
        sa.lpSecurityDescriptor = &sd;

        if(!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))
        {
            DBWinErrorMessage("Unable to initialize Security Descriptor");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }
        if(!SetSecurityDescriptorDacl(&sd, TRUE, (PACL)NULL, FALSE))
        {
            DBWinErrorMessage("Unable to set Security Descriptor");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        hAckEvent = CreateEvent(&sa, FALSE, FALSE, "DBWIN_BUFFER_READY");
        if (!hAckEvent)
        {
            DBWinErrorMessage("Unable to create synchronization object DBWIN_BUFFER_READY");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        if (GetLastError() == ERROR_ALREADY_EXISTS)
        {
            DBWinErrorMessage("Already running");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        g_hReadyEvent = CreateEvent(&sa, FALSE, FALSE, "DBWIN_DATA_READY");
        if (!g_hReadyEvent)
        {
            DBWinErrorMessage("Unable to create synchronization object DBWIN_DATA_READY");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        SharedFile = CreateFileMapping((HANDLE)-1, &sa, PAGE_READWRITE, 0, 4096, "DBWIN_BUFFER");
        if (!SharedFile)
        {
            DBWinErrorMessage("Unable to create file mapping object DBWIN_BUFFER");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        SharedMem = MapViewOfFile(SharedFile, FILE_MAP_READ, 0, 0, 512);
        if (!SharedMem)
        {
            DBWinErrorMessage("Unable to map shared memory");
            RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
        }

        lpszString = (LPSTR)SharedMem + sizeof(DWORD);
        pThisPid = (LPDWORD)SharedMem;
        LastPid = 0xffffffff;
        SetEvent(hAckEvent);

        _active = true;
		for (;;)
        {
            ret = WaitForSingleObject(g_hReadyEvent, INFINITE);

            /* leave if we are no longer wanted */
			if ( _active == false) {
                break;
			}

            if (ret != WAIT_OBJECT_0)
            {
                DBWinErrorMessage("Wait failed");
                RaiseException(EXCEPT_LEAVE_EVENT_THREAD, 0, 0, NULL);
            }
            else
            {
				if (TraceMonitorMessage != NULL) {
					// Send the message via BeginInvoke to avoid any delays in capturing the next message
					TraceMonitorMessage->BeginInvoke(this, new TraceMonitorEventArgs(*pThisPid, lpszString), NULL, NULL);
				}
                /* Ready for new event */
                SetEvent(hAckEvent);
            }
        }
    }
	__except( GetExceptionCode() == EXCEPT_LEAVE_EVENT_THREAD)
    {
        /* nothing to do here */
		System::Environment::Exit(1);
    }

    if ( hAckEvent != INVALID_HANDLE_VALUE )
        SetEvent(hAckEvent);

    if (hAckEvent != INVALID_HANDLE_VALUE)
        CloseHandle(hAckEvent);

    if (SharedMem != NULL)
        UnmapViewOfFile(SharedMem);

    if (SharedFile != INVALID_HANDLE_VALUE)
        CloseHandle(SharedFile);
}

