using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace TraceMonitor {
   /// <summary>
   /// Dialog box to show the filtered words or programs, and allow the user to delete them.
   /// </summary>
   public class FilteredList : System.Windows.Forms.Form {
      ArrayList filteredPrograms;
      private System.Windows.Forms.Button okButton;
      private System.Windows.Forms.Button cancelButton;
      private System.Windows.Forms.ListView filterList;
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;

      public FilteredList(ArrayList filteredPrograms) {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

			
         this.filteredPrograms = filteredPrograms;

         foreach (String s in filteredPrograms) {
            filterList.Items.Add(new ListViewItem(s));
         }
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

		#region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         this.filterList = new System.Windows.Forms.ListView();
         this.okButton = new System.Windows.Forms.Button();
         this.cancelButton = new System.Windows.Forms.Button();
         this.SuspendLayout();
         // 
         // filterList
         // 
         this.filterList.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
            | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right);
         this.filterList.Font = new System.Drawing.Font("Lucida Console", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
         this.filterList.Location = new System.Drawing.Point(8, 8);
         this.filterList.Name = "filterList";
         this.filterList.Size = new System.Drawing.Size(496, 240);
         this.filterList.TabIndex = 0;
         this.filterList.View = System.Windows.Forms.View.List;
         this.filterList.KeyDown += new System.Windows.Forms.KeyEventHandler(this.listView1_KeyDown);
         // 
         // okButton
         // 
         this.okButton.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right);
         this.okButton.DialogResult = System.Windows.Forms.DialogResult.OK;
         this.okButton.Location = new System.Drawing.Point(151, 256);
         this.okButton.Name = "okButton";
         this.okButton.TabIndex = 1;
         this.okButton.Text = "OK";
         this.okButton.Click += new System.EventHandler(this.okButton_Click);
         // 
         // cancelButton
         // 
         this.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
         this.cancelButton.Location = new System.Drawing.Point(287, 256);
         this.cancelButton.Name = "cancelButton";
         this.cancelButton.TabIndex = 2;
         this.cancelButton.Text = "Cancel";
         // 
         // FilteredList
         // 
         this.AcceptButton = this.okButton;
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.CancelButton = this.cancelButton;
         this.ClientSize = new System.Drawing.Size(512, 294);
         this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                      this.cancelButton,
                                                                      this.okButton,
                                                                      this.filterList});
         this.Name = "FilteredList";
         this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
         this.Text = "FilteredList";
         this.ResumeLayout(false);

      }
		#endregion

      /// <summary>
      /// When the user clicks ok, add any programs that are still in the list.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void okButton_Click(object sender, System.EventArgs e) {
         filteredPrograms.Clear();
         foreach (ListViewItem lvi in filterList.Items) {
            filteredPrograms.Add(lvi.Text);
         }
      }

      /// <summary>
      /// If the key is delete, remove the focused item from the list.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void listView1_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) {
         if (e.KeyCode == Keys.Delete && filterList.FocusedItem != null) {
            filteredPrograms.Remove(filterList.FocusedItem.Text);
            filterList.FocusedItem.Remove();
         }
      }

      /// <summary>
      /// Text in the ArrayList of filtered programs
      /// </summary>
      public ArrayList FilteredPrograms {
         get { return FilteredPrograms; }
      }
   }
}
