using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace TraceMonitor {
   /// <summary>
   /// Modal dialog box to display individual messages in a mutli-line text box.
   /// </summary>
   public class MessageDetail : System.Windows.Forms.Form {
      private System.Windows.Forms.Label label1;
      private System.Windows.Forms.Label processId;
      private System.Windows.Forms.Button closeButton;
      private System.Windows.Forms.Button upButton;
      private System.Windows.Forms.Button downButton;
      /// <summary>
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;
      private System.Windows.Forms.Label processTime;
      private System.Windows.Forms.Label label3;
      private System.Windows.Forms.TextBox message;
      private System.Windows.Forms.Label processName;
      private System.Windows.Forms.Label label4;

      IViewableList _list;

      /// <summary>
      /// Constructor with the first message to display.
      /// </summary>
      /// <param name="processId">Process ID the message was sent from</param>
      /// <param name="processName">Process Name the message was sent from.</param>
      /// <param name="messageTime">Time of the message (is ms from process start)</param>
      /// <param name="message">Actual text of the message.</param>
      /// <param name="list">IViewable list interface to allow the dialog box to retrieve other messages.</param>
      public MessageDetail(int processId, String processName, String messageTime, String message, IViewableList list) {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         _list = list;

         this.processId.Text = processId.ToString();
         this.processName.Text = processName;
         this.processTime.Text = messageTime;
         this.message.Text = message;
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

		#region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MessageDetail));
         this.label1 = new System.Windows.Forms.Label();
         this.processId = new System.Windows.Forms.Label();
         this.message = new System.Windows.Forms.TextBox();
         this.closeButton = new System.Windows.Forms.Button();
         this.upButton = new System.Windows.Forms.Button();
         this.downButton = new System.Windows.Forms.Button();
         this.processTime = new System.Windows.Forms.Label();
         this.label3 = new System.Windows.Forms.Label();
         this.processName = new System.Windows.Forms.Label();
         this.label4 = new System.Windows.Forms.Label();
         this.SuspendLayout();
         // 
         // label1
         // 
         this.label1.Location = new System.Drawing.Point(8, 8);
         this.label1.Name = "label1";
         this.label1.TabIndex = 0;
         this.label1.Text = "Process ID:";
         this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // processId
         // 
         this.processId.Location = new System.Drawing.Point(112, 8);
         this.processId.Name = "processId";
         this.processId.TabIndex = 1;
         this.processId.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // message
         // 
         this.message.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
            | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right);
         this.message.Location = new System.Drawing.Point(8, 112);
         this.message.Multiline = true;
         this.message.Name = "message";
         this.message.ReadOnly = true;
         this.message.ScrollBars = System.Windows.Forms.ScrollBars.Both;
         this.message.Size = new System.Drawing.Size(560, 176);
         this.message.TabIndex = 2;
         this.message.Text = "";
         this.message.WordWrap = false;
         // 
         // closeButton
         // 
         this.closeButton.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right);
         this.closeButton.DialogResult = System.Windows.Forms.DialogResult.OK;
         this.closeButton.Location = new System.Drawing.Point(238, 296);
         this.closeButton.Name = "closeButton";
         this.closeButton.Size = new System.Drawing.Size(101, 23);
         this.closeButton.TabIndex = 3;
         this.closeButton.Text = "Close";
         // 
         // upButton
         // 
         this.upButton.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
         this.upButton.Image = ((System.Drawing.Bitmap)(resources.GetObject("upButton.Image")));
         this.upButton.Location = new System.Drawing.Point(544, 8);
         this.upButton.Name = "upButton";
         this.upButton.Size = new System.Drawing.Size(24, 23);
         this.upButton.TabIndex = 4;
         this.upButton.Click += new System.EventHandler(this.upButton_Click);
         // 
         // downButton
         // 
         this.downButton.Anchor = (System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right);
         this.downButton.Image = ((System.Drawing.Bitmap)(resources.GetObject("downButton.Image")));
         this.downButton.Location = new System.Drawing.Point(544, 40);
         this.downButton.Name = "downButton";
         this.downButton.Size = new System.Drawing.Size(24, 23);
         this.downButton.TabIndex = 5;
         this.downButton.Click += new System.EventHandler(this.downButton_Click);
         // 
         // processTime
         // 
         this.processTime.Location = new System.Drawing.Point(112, 72);
         this.processTime.Name = "processTime";
         this.processTime.TabIndex = 7;
         this.processTime.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // label3
         // 
         this.label3.Location = new System.Drawing.Point(8, 72);
         this.label3.Name = "label3";
         this.label3.TabIndex = 6;
         this.label3.Text = "Process Time:";
         this.label3.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // processName
         // 
         this.processName.Location = new System.Drawing.Point(112, 40);
         this.processName.Name = "processName";
         this.processName.TabIndex = 9;
         this.processName.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // label4
         // 
         this.label4.Location = new System.Drawing.Point(8, 40);
         this.label4.Name = "label4";
         this.label4.TabIndex = 8;
         this.label4.Text = "Process Name:";
         this.label4.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // MessageDetail
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(576, 326);
         this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                      this.processName,
                                                                      this.label4,
                                                                      this.processTime,
                                                                      this.label3,
                                                                      this.downButton,
                                                                      this.upButton,
                                                                      this.closeButton,
                                                                      this.message,
                                                                      this.processId,
                                                                      this.label1});
         this.Name = "MessageDetail";
         this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
         this.Text = "MessageDetail";
         this.ResumeLayout(false);

      }
		#endregion

      /// <summary>
      /// Fetch the previous messagge in the list.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void upButton_Click(object sender, System.EventArgs e) {
         String messageTime;
         String message;
         if (_list.getPreviousMessage(out messageTime, out message) ) {
            this.message.Text = message;
            this.processTime.Text = messageTime;
         }
      }

      /// <summary>
      /// Fetch the next message in the list.
      /// </summary>
      /// <param name="sender"></param>
      /// <param name="e"></param>
      private void downButton_Click(object sender, System.EventArgs e) {
         String messageTime;
         String message;
         if (_list.getNextMessage(out messageTime, out message) ) {
            this.processTime.Text = messageTime;
            this.message.Text = message;
         }
      }
   }
}
