using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;

namespace TraceMonitor {
   /// <summary>
   /// Class to store all the options for the form (color, font, max messge count, etc)
   /// </summary>
   [Serializable]
   [SzurgotUtilities.SaveProperties]
   public class Options {
      public Options() { }

      private Color _processListBackColor = Color.White;
      [Description("Background color for the process list")]
      [Category("Process List")]
      public Color ProcessListBackColor {
         get { return _processListBackColor; }
         set { _processListBackColor = value; }
      }

      private Color _processListForeColor = Color.Black;
      [Description("Font color for the process list")]
      [Category("Process List")]
      public Color ProcessListForeColor {
         get { return _processListForeColor; }
         set { _processListForeColor = value; }
      }

      private Font _processListFont = new Font("Courier New", 8);
      [Description("Font for the process list")]
      [Category("Process List")]
      public Font ProcessListFont {
         get { return _processListFont; }
         set { _processListFont = value; }
      }

      private Color _messageListBackColor = Color.Black;
      [Description("Background color for the process list")]
      [Category("Message List")]
      public Color MessageListBackColor {
         get { return _messageListBackColor; }
         set { _messageListBackColor = value; }
      }

      private Color _messageListForeColor = Color.White;
      [Description("Font color for the process list")]
      [Category("Message List")]
      public Color MessageListForeColor {
         get { return _messageListForeColor; }
         set { _messageListForeColor = value; }
      }

      private Font _messageListFont = new Font("Courier New", 8);
      [Description("Font for the process list")]
      [Category("Message List")]
      public Font MessageListFont {
         get { return _messageListFont; }
         set { _messageListFont = value; }
      }

      private int _maxClosedPrograms = 10;
      [Description("As programs are closed, when the count reaches this threshhold the oldest will be removed.")]
      public int MaxClosedPrograms {
         get { return _maxClosedPrograms; }
         set { _maxClosedPrograms = value; }
      }

      private bool _confirmQuit = false;
      [Description("Should the program confirm quit when the window close box is clicked?")]
      public bool ConfirmQuit {
         get { return _confirmQuit; }
         set { _confirmQuit = value; }
      }

      private int _maxMessagesForAll = 1000;
      [Description("How many messages to show in the aggregate window")]
      public int MaxMessagesForAll {
         get { return _maxMessagesForAll; }
         set { _maxMessagesForAll = value; }
      }


      // If a new instance of this program appears, we automatically focus on it...
      private String _monitoredProgram = null;
      [Browsable(false)]
      public String MonitoredProgram {
         get { return _monitoredProgram; }
         set { _monitoredProgram = value; }
      }

      // If an instance of any of these programs appear, disregard them
      private ArrayList _filteredPrograms = new ArrayList();
      [Browsable(false)]
      public ArrayList FilteredPrograms {
         get { return _filteredPrograms; }
         set { _filteredPrograms = value; }
      }

      // if a message mateches any of these regular expressions, disregard it
      private ArrayList _filteredWords = new ArrayList();
      [Browsable(false)]
      public ArrayList FilteredWords {
         get { return _filteredWords; }
         set { _filteredWords = value; }
      }

      // When minimized, should it remain in the task bar or minimize to the tray
      private bool _minimizeToTray = false;
      [Browsable(false)]
      public bool MinimizeToTray {
         get { return _minimizeToTray; }
         set { _minimizeToTray = value; }
      }

      // Stores the bounds of the forms to restore at the next run
      private Rectangle _formBounds = Rectangle.Empty;
      [Browsable(false)]
      public Rectangle FormBounds {
         get { return _formBounds; }
         set { _formBounds = value; }
      }

      // Stores the placement of the splitter
      private int _splitterPosition = -1;
      [Browsable(false)]
      public int SplitterPosition {
         get { return _splitterPosition; }
         set { _splitterPosition = value; }
      }

      private double _opacity = 100;
      [Browsable(false)]
      public double Opacity {
         get { return _opacity; }
         set { _opacity = value; }
      }
   }
}
