using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace TraceMonitor {
   /// <summary>
   /// When the process list is minimized, this User control is used to display the process name and ID vertically.
   /// </summary>
   public class ProcessNamePanel : System.Windows.Forms.UserControl {
      
      private String _currentProcessName;
      /// <summary>
      /// Allows the program to change the current process name if the current process changes. (Via monitored program instantiation)
      /// </summary>
      public String CurrentProcessName {
         get { return _currentProcessName; }
         set { _currentProcessName = value; }
      }
      
      /// <summary> 
      /// Required designer variable.
      /// </summary>
      private System.ComponentModel.Container components = null;

      public ProcessNamePanel(String currentProcessName) {
         // This call is required by the Windows.Forms Form Designer.
         InitializeComponent();

         _currentProcessName = currentProcessName;
      }

      /// <summary> 
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if(components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

		#region Component Designer generated code
      /// <summary> 
      /// Required method for Designer support - do not modify 
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
         // 
         // ProcessPanel
         // 
         this.Name = "ProcessPanel";
         this.Size = new System.Drawing.Size(24, 264);

      }
		#endregion

      /// <summary>
      /// Transform and paint the process name vertically.
      /// </summary>
      /// <param name="e"></param>
      protected override void OnPaint(PaintEventArgs e) {
         e.Graphics.FillRectangle(Brushes.White, 0, 0, this.Width, this.Height);
         e.Graphics.DrawRectangle(Pens.Black, 0, 0, this.Width, this.Height);
         e.Graphics.RotateTransform(-90);
         Font f = new Font("Courier New", 16, FontStyle.Bold);

         SizeF f1 = e.Graphics.MeasureString(CurrentProcessName, f);
         int loc = (int)((this.Bounds.Height / 2) + (f1.Width / 2));
         
         e.Graphics.DrawString(CurrentProcessName, f, Brushes.Black, -loc, 0);

      }
   }
}
