// TODO Maintain state on "Monitor Selected Program Menu" Item
// TODO When filtered program is removed from filter list, actually redisplay it :)
// TODO display multiline items as mutliline :)

using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

using System.IO;
using System.Reflection;
using System.Xml;
using System.Text.RegularExpressions;
using System.Configuration;
using System.Text;

using System.Runtime.Serialization.Formatters.Binary;

namespace TraceMonitor {

   /// <summary>
   /// Main form and code body for the TraceMonitor.
   /// </summary>
   public class TraceMonitorForm : System.Windows.Forms.Form, IViewableList {
      // List of all the programs that have sent trace outputs, and haven't been removed from the list.
      Hashtable activeDisplays = new Hashtable();
      // Thread for the actual trace monitor (C++ dll)
      Thread monitorThread;
      // Active process id. Defaults to -1 to allow the first program to display.
      int activeProcessId = -1;
      // Count of terminated programs.
      int terminatedCount = 0;

      // Main title of the form. The process name will be appended as it changes.
      private String mainTitle = "Trace Output Monitor";

      private String optionsPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "SzurgotDevelopment\\TraceMonitor.config");

      Options options;

      Icon standardIcon;
      Icon alertIcon;

      Size processListSize;
      Size messageListSize;

      // Collate all program input
      WatchedProcessInfo allTraces;

      bool flashingIcon = false;

      // Is the process list panel closed
      bool processListClosed = false;

      // Since the column count changes, which column is the message (for detail display)
      int messageColumn = 1;

      // The actual function to listen for the traces. Created from a C++ dll.
      DbMonClass.DbMon dbMon = new DbMonClass.DbMon();

      Queue messageQueue = new Queue();

      bool messageListInUpdate = false;

      String indent = null;

		bool loggingOff = false;

		MenuItem contextVisibilityMenu;

		// Whether the form is the current window or not
		// TODO Probably a better way to do this
		bool activated = false;

      private System.Windows.Forms.Splitter splitter1;
      private System.Windows.Forms.MainMenu mainMenu1;
      private System.Windows.Forms.MenuItem menuItem1;
      private System.Windows.Forms.MenuItem menuItem3;
      private System.Windows.Forms.ColumnHeader pid;
      private System.Windows.Forms.ColumnHeader desc;
      private System.Windows.Forms.MenuItem exitMenu;
      private System.Windows.Forms.ListView messageList;
      private System.Windows.Forms.ColumnHeader message;
      private System.Windows.Forms.ListView processList;
      private System.Windows.Forms.ColumnHeader procTime;
      private System.Windows.Forms.MenuItem menuItem2;
      private System.Windows.Forms.MenuItem infoMenu;
      private System.Windows.Forms.MenuItem sendTraceMenu;
      private System.Windows.Forms.MenuItem optionsMenu;
      private System.Windows.Forms.Panel processListPanel;
      private System.Windows.Forms.MenuItem filterSelected;
      private System.Windows.Forms.MenuItem menuItem4;
      private System.Windows.Forms.Button chevronButton;
      private System.Windows.Forms.MenuItem menuItem5;
      private System.Windows.Forms.MenuItem monitorProgramMenu;
      private System.Windows.Forms.NotifyIcon notifyIcon1;
      private System.Windows.Forms.MenuItem minimizeToTray;
      private System.Windows.Forms.MenuItem showFilteredProgramList;
      private System.Windows.Forms.MenuItem showFillteredWordList;
      private System.Windows.Forms.MenuItem addFilteredWord;
      private System.Windows.Forms.MenuItem clearTraces;
      private System.Windows.Forms.MenuItem menuItem7;
      private System.Windows.Forms.MenuItem copyMenu;
      private System.Windows.Forms.MenuItem menuItem8;
      private System.Windows.Forms.MenuItem selectAllMenu;
      private System.Windows.Forms.MenuItem saveTraceMenu;
      private System.Windows.Forms.MenuItem menuItem10;
      private System.Windows.Forms.ContextMenu contextMenu1;
      private System.Windows.Forms.MenuItem closeFromIcon;
		private System.Windows.Forms.MenuItem hideRestoreMenu;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem aboutMenu;
      private System.Windows.Forms.ContextMenu itemContextMenu;
      private System.Windows.Forms.MenuItem killSelectedProcess;
      private System.Windows.Forms.MenuItem filterSelectedProgramContext;
      private System.Windows.Forms.MenuItem monitorSelectedProgramContext;
      private System.Windows.Forms.MenuItem killSelectedProgramContext;
		private System.Windows.Forms.MenuItem visibilityMenu;
		private System.Windows.Forms.MenuItem visible100menu;
		private System.Windows.Forms.MenuItem visible90menu;
		private System.Windows.Forms.MenuItem visible80menu;
		private System.Windows.Forms.MenuItem visible70menu;
		private System.Windows.Forms.MenuItem visible60menu;
		private System.Windows.Forms.MenuItem visible50menu;
		private System.Windows.Forms.MenuItem visible40menu;
		private System.Windows.Forms.MenuItem visible30menu;
		private System.Windows.Forms.MenuItem visible20menu;
		private System.Windows.Forms.MenuItem visible10menu;
      private System.ComponentModel.IContainer components;

      public TraceMonitorForm() {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         // Grab the icons
         Assembly assm = Assembly.GetExecutingAssembly();
         standardIcon = new Icon(assm.GetManifestResourceStream("TraceMonitor.Images.STANDARD.ICO"));
         alertIcon = new Icon(assm.GetManifestResourceStream("TraceMonitor.Images.NEWALERT.ICO"));

         // Setup to actually list for the trace message
         dbMon.TraceMonitorMessage += new DbMonClass.TraceMonitorEventHandler(OnTraceMonitorMessage);

         options = (Options)SzurgotUtilities.SaveProperties.LoadObject();
         if (options == null) {
            options = new Options();
         }

         setOptions();

         this.Text = mainTitle;

         processListSize = processList.Size;
         messageListSize = messageList.Size;

         allTraces = new WatchedProcessInfo(0, "All Traces", DateTime.Now);

			this.SetStyle(ControlStyles.EnableNotifyMessage, true);

			contextVisibilityMenu = visibilityMenu.CloneMenu();

			contextMenu1.MenuItems.Add(0, contextVisibilityMenu);
      }

      // The actual thread running the trace listener
      private void MonitorThreadProcess() {
         dbMon.EventThreadRoutine();
      }

      private void setOptions() {
         processList.BackColor = options.ProcessListBackColor;
         processList.ForeColor = options.ProcessListForeColor;
         processList.Font = options.ProcessListFont;

         messageList.BackColor = options.MessageListBackColor;
         messageList.ForeColor = options.MessageListForeColor;
         messageList.Font = options.MessageListFont;

         if (options.MinimizeToTray == true) {
            minimizeToTray_Click(this, EventArgs.Empty);
         }

         if (options.SplitterPosition != -1) {
            splitter1.SplitPosition = options.SplitterPosition;
         }

			this.Opacity = options.Opacity;
         // Bounds and splitter handled at visible
      }

      // Event from the dbmon class. This marshals the event back to the main thread.
      private void OnTraceMonitorMessage(Object sender, DbMonClass.TraceMonitorEventArgs e) {
         // Queue it so we can see if there's a backlog of messages to process.
         messageQueue.Enqueue(e);
         TraceMonitorMessageDelegate tmm = new TraceMonitorMessageDelegate(OnTraceMonitorMessageMarshal);
         this.BeginInvoke(tmm, new Object[]{});
      }

      
      // Receives the trace events, and decides how best to handle it.
      private void OnTraceMonitorMessageMarshal() {
         DateTime start = DateTime.Now;
         lock (this) {
            DbMonClass.TraceMonitorEventArgs e = (DbMonClass.TraceMonitorEventArgs)
               messageQueue.Dequeue();

            if (e.Message.Trim() == "") {
               indent = e.Message;
               return;
            }

            String message = e.Message;
            if (indent != null) {
               message = indent + message;
               indent = null;
            }

            if (messageQueue.Count > 0 && !messageListInUpdate) {
               messageList.BeginUpdate();
               messageListInUpdate = true;
            }
            WatchedProcessInfo wpi;
            if (activeDisplays.ContainsKey(e.ProcessId)) {
               // Are we already watching this process?
               wpi = (WatchedProcessInfo)activeDisplays[e.ProcessId];
            } else {
               // If not, add a new watched process. If we can't retreive the name, the name becomes the ID.
               DateTime startTime = DateTime.Now;
               String name = "";
               String moduleName = "";
               try {
                  Process p = Process.GetProcessById(e.ProcessId);
                  name = p.ProcessName;
                  startTime = p.StartTime;
                  moduleName = p.ProcessName;
                  p.Exited += new EventHandler(onProcessExited);
                  p.EnableRaisingEvents = true;
               } catch (InvalidOperationException) {
                  name = e.ProcessId.ToString();
               } catch (System.ComponentModel.Win32Exception) {
                  name = e.ProcessId.ToString();
               } catch (Exception) {
                  name = e.ProcessId.ToString();
               }
               wpi = new WatchedProcessInfo(e.ProcessId, name, startTime);

               activeDisplays.Add(e.ProcessId, wpi);

               // Should we filter it?
               if ( options.FilteredPrograms.Contains(name) ) {
                  wpi.Filtered = true;
               }
            }

            if (!wpi.Filtered) {
               // Filter out any regular expressions here
               if (filterMessage(message) == true) {
                  return;
               }

               if (!wpi.OutputShown) {
                  ListViewItem lvi = new ListViewItem(new String[2]{e.ProcessId.ToString(), wpi.DisplayName});
                  lvi.Tag = wpi;
                  wpi.ListItem = lvi; // Circular Reference


                  // Add it so that it's at the top of the list just below the all traces
                  processList.Items.Insert(1, lvi);

                  // See if it's the watched program
                  if (options.MonitoredProgram != null &&  wpi.DisplayName.ToLower() == options.MonitoredProgram.ToLower()) {
                     try {
                        processList.SelectedItems.Clear();
                  
                        lvi.Focused = true;
                        lvi.Selected = true;
                     } catch (Exception ex) {
                        MessageBox.Show("Exception\n" + ex);
                     }
                  }

                  wpi.OutputShown = true;
               }

               MessageInfo mi = wpi.AppendMessage(message);
					// TODO Should probably make sure icon is there...
               if ( (this.Visible == false || !activated) && !flashingIcon) {
                  flashingIcon = true;
                  System.Threading.ThreadPool.QueueUserWorkItem(new WaitCallback(flashIcon));
                  notifyIcon1.Icon = alertIcon;
               }

               // Add it to the all traces
               allTraces.AppendMessage(message, e.ProcessId, wpi.ProcessStartTime);

               lock (messageList) {
                  if (activeProcessId == e.ProcessId || activeProcessId == 0) {
                     ListViewItem lvi;
                     if (activeProcessId == 0) {
                        lvi = new ListViewItem(new String[3]{mi.SecondsSinceStart, e.ProcessId.ToString(), mi.Message});
                     } else {
                        lvi = new ListViewItem(new String[2]{mi.SecondsSinceStart, mi.Message});
                     }
                     messageList.Items.Add(lvi);

                     // Calc the average size and remove the header row from the line count
                     try {
                        if (this.Visible) {
                           int topItemHeight = lvi.Bounds.Height;
                           int viewedLines = (int)((float)messageList.DisplayRectangle.Height / (float)topItemHeight) - 1;

                           int linesFromBottom = messageList.Items.Count - messageList.TopItem.Index;
                  
                           if (linesFromBottom <= viewedLines + 2) {
                              lvi.EnsureVisible();
                           }
                        }
                     } catch (Exception ex){ 
                        Trace.WriteLine("Exception:" + ex);
                     }
                  }
               }
            }

            if (messageQueue.Count == 0 && messageListInUpdate) {
               messageList.EndUpdate();
               messageListInUpdate = false;
            }

            // Done this way so if a lot of messages come at once, it's not spending most of it's time updating the UI.
            if (messageQueue.Count == 0 && allTraces.Messages.Count > options.MaxMessagesForAll) {
               if (activeProcessId == 0) {
                  messageList.BeginUpdate();
               }
               // For all traces, remove the oldest ones
               while (allTraces.Messages.Count > options.MaxMessagesForAll) {
                  allTraces.Messages.RemoveAt(0);
                  if (activeProcessId == 0) {
                     messageList.Items.RemoveAt(0);
                  }
               }
               if (activeProcessId == 0) {
                  messageList.EndUpdate();
               }
            }
         }
      }

      // If the form is minimized to the tray, flash the tray icon to inform of new trace messages.
      private void flashIcon(Object state) {
         bool showAlert = false;
         while (flashingIcon == true) {
            showAlert = !showAlert;
            if (showAlert) {
               notifyIcon1.Icon = alertIcon;
            } else {
               notifyIcon1.Icon = standardIcon;
            }
            Thread.Sleep(500);
         }
			notifyIcon1.Icon = standardIcon;
         flashingIcon = false;
      }

      // Checks to see if the message should be displayed.
      private bool filterMessage(String message) {
         String msg = message.Trim();

         foreach (String s in options.FilteredWords) {
            if (Regex.IsMatch(msg, s) ) {
               return true;
            }
         }
         return false;
      }

      // When the process exits, we stop worrying about it, and move it to below any active programs.
      private void onProcessExited(Object sender, EventArgs e) {
         lock (this) {
            try {
               Process p = (Process)sender;
               WatchedProcessInfo wpi = (WatchedProcessInfo)activeDisplays[p.Id];
               activeDisplays.Remove(p.Id);

               if (wpi != null && wpi.OutputShown) {
                  bool keepFocus = wpi.ListItem.Focused;
                  if (keepFocus) {
                     // remove the focus so no other items gets it while we're moving this one
                     wpi.ListItem.Focused = false;
                  }
                  processList.SelectedItems.Clear();
                  wpi.ListItem.Remove();
                  wpi.ListItem.SubItems[1].Text = "* " + wpi.ListItem.SubItems[1].Text;
                  processList.Items.Insert(processList.Items.Count - terminatedCount, wpi.ListItem);
                  terminatedCount++;
                  if (keepFocus) {
                     wpi.ListItem.Focused = true;
                     wpi.ListItem.Selected = true;
                     changeDisplayedItem();
                  }
               }

               if (terminatedCount > options.MaxClosedPrograms) {
                  // Remove the last item
                  processList.Items.RemoveAt(processList.Items.Count - 1);
                  terminatedCount--;
               }
            } catch (Exception ex) {
               MessageBox.Show("EX:" + ex);
            }
         }
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing ) {
         if( disposing ) {
            if (components != null) {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

		#region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TraceMonitorForm));
			this.processList = new System.Windows.Forms.ListView();
			this.pid = new System.Windows.Forms.ColumnHeader();
			this.desc = new System.Windows.Forms.ColumnHeader();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.saveTraceMenu = new System.Windows.Forms.MenuItem();
			this.menuItem10 = new System.Windows.Forms.MenuItem();
			this.exitMenu = new System.Windows.Forms.MenuItem();
			this.menuItem7 = new System.Windows.Forms.MenuItem();
			this.copyMenu = new System.Windows.Forms.MenuItem();
			this.menuItem8 = new System.Windows.Forms.MenuItem();
			this.selectAllMenu = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.showFilteredProgramList = new System.Windows.Forms.MenuItem();
			this.showFillteredWordList = new System.Windows.Forms.MenuItem();
			this.minimizeToTray = new System.Windows.Forms.MenuItem();
			this.clearTraces = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.addFilteredWord = new System.Windows.Forms.MenuItem();
			this.optionsMenu = new System.Windows.Forms.MenuItem();
			this.visibilityMenu = new System.Windows.Forms.MenuItem();
			this.visible100menu = new System.Windows.Forms.MenuItem();
			this.visible90menu = new System.Windows.Forms.MenuItem();
			this.visible80menu = new System.Windows.Forms.MenuItem();
			this.visible70menu = new System.Windows.Forms.MenuItem();
			this.visible60menu = new System.Windows.Forms.MenuItem();
			this.visible50menu = new System.Windows.Forms.MenuItem();
			this.visible40menu = new System.Windows.Forms.MenuItem();
			this.visible30menu = new System.Windows.Forms.MenuItem();
			this.visible20menu = new System.Windows.Forms.MenuItem();
			this.visible10menu = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.filterSelected = new System.Windows.Forms.MenuItem();
			this.monitorProgramMenu = new System.Windows.Forms.MenuItem();
			this.killSelectedProcess = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.infoMenu = new System.Windows.Forms.MenuItem();
			this.sendTraceMenu = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.aboutMenu = new System.Windows.Forms.MenuItem();
			this.messageList = new System.Windows.Forms.ListView();
			this.procTime = new System.Windows.Forms.ColumnHeader();
			this.message = new System.Windows.Forms.ColumnHeader();
			this.processListPanel = new System.Windows.Forms.Panel();
			this.chevronButton = new System.Windows.Forms.Button();
			this.notifyIcon1 = new System.Windows.Forms.NotifyIcon(this.components);
			this.contextMenu1 = new System.Windows.Forms.ContextMenu();
			this.hideRestoreMenu = new System.Windows.Forms.MenuItem();
			this.menuItem11 = new System.Windows.Forms.MenuItem();
			this.closeFromIcon = new System.Windows.Forms.MenuItem();
			this.itemContextMenu = new System.Windows.Forms.ContextMenu();
			this.filterSelectedProgramContext = new System.Windows.Forms.MenuItem();
			this.monitorSelectedProgramContext = new System.Windows.Forms.MenuItem();
			this.killSelectedProgramContext = new System.Windows.Forms.MenuItem();
			this.processListPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// processList
			// 
			this.processList.BackColor = System.Drawing.Color.White;
			this.processList.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																												  this.pid,
																												  this.desc});
			this.processList.Dock = System.Windows.Forms.DockStyle.Fill;
			this.processList.Font = new System.Drawing.Font("Courier New", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.processList.FullRowSelect = true;
			this.processList.GridLines = true;
			this.processList.HideSelection = false;
			this.processList.Location = new System.Drawing.Point(0, 0);
			this.processList.Name = "processList";
			this.processList.Size = new System.Drawing.Size(224, 478);
			this.processList.TabIndex = 1;
			this.processList.View = System.Windows.Forms.View.Details;
			this.processList.KeyDown += new System.Windows.Forms.KeyEventHandler(this.processList_KeyDown);
			this.processList.MouseUp += new System.Windows.Forms.MouseEventHandler(this.processList_MouseUp);
			this.processList.SelectedIndexChanged += new System.EventHandler(this.processList_SelectedIndexChanged);
			// 
			// pid
			// 
			this.pid.Text = "PID";
			this.pid.Width = 34;
			// 
			// desc
			// 
			this.desc.Text = "Name";
			this.desc.Width = 183;
			// 
			// splitter1
			// 
			this.splitter1.Location = new System.Drawing.Point(224, 0);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 478);
			this.splitter1.TabIndex = 2;
			this.splitter1.TabStop = false;
			this.splitter1.SplitterMoved += new System.Windows.Forms.SplitterEventHandler(this.splitter1_SplitterMoved);
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.menuItem1,
																											 this.menuItem7,
																											 this.menuItem2,
																											 this.menuItem5,
																											 this.menuItem3});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.saveTraceMenu,
																											 this.menuItem10,
																											 this.exitMenu});
			this.menuItem1.Text = "&File";
			// 
			// saveTraceMenu
			// 
			this.saveTraceMenu.Index = 0;
			this.saveTraceMenu.Text = "&Save Current Trace";
			this.saveTraceMenu.Click += new System.EventHandler(this.saveTraceMenu_Click);
			// 
			// menuItem10
			// 
			this.menuItem10.Index = 1;
			this.menuItem10.Text = "-";
			// 
			// exitMenu
			// 
			this.exitMenu.Index = 2;
			this.exitMenu.Text = "E&xit";
			this.exitMenu.Click += new System.EventHandler(this.exitMenu_Click);
			// 
			// menuItem7
			// 
			this.menuItem7.Index = 1;
			this.menuItem7.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.copyMenu,
																											 this.menuItem8,
																											 this.selectAllMenu});
			this.menuItem7.Text = "&Edit";
			// 
			// copyMenu
			// 
			this.copyMenu.Index = 0;
			this.copyMenu.Shortcut = System.Windows.Forms.Shortcut.CtrlC;
			this.copyMenu.Text = "&Copy";
			this.copyMenu.Click += new System.EventHandler(this.copyMenu_Click);
			// 
			// menuItem8
			// 
			this.menuItem8.Index = 1;
			this.menuItem8.Text = "-";
			// 
			// selectAllMenu
			// 
			this.selectAllMenu.Index = 2;
			this.selectAllMenu.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.selectAllMenu.Text = "Select &All";
			this.selectAllMenu.Click += new System.EventHandler(this.selectAllMenu_Click);
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 2;
			this.menuItem2.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.showFilteredProgramList,
																											 this.showFillteredWordList,
																											 this.minimizeToTray,
																											 this.clearTraces,
																											 this.menuItem4,
																											 this.addFilteredWord,
																											 this.optionsMenu,
																											 this.visibilityMenu});
			this.menuItem2.Text = "&Tools";
			// 
			// showFilteredProgramList
			// 
			this.showFilteredProgramList.Index = 0;
			this.showFilteredProgramList.Text = "Show Filtered &Program List";
			this.showFilteredProgramList.Click += new System.EventHandler(this.showFilteredProgramList_Click);
			// 
			// showFillteredWordList
			// 
			this.showFillteredWordList.Index = 1;
			this.showFillteredWordList.Text = "Show Filtered &Word List";
			this.showFillteredWordList.Click += new System.EventHandler(this.showFillteredWordList_Click);
			// 
			// minimizeToTray
			// 
			this.minimizeToTray.Index = 2;
			this.minimizeToTray.Text = "&Minimize to tray";
			this.minimizeToTray.Click += new System.EventHandler(this.minimizeToTray_Click);
			// 
			// clearTraces
			// 
			this.clearTraces.Index = 3;
			this.clearTraces.Shortcut = System.Windows.Forms.Shortcut.CtrlT;
			this.clearTraces.Text = "&Clear Traces";
			this.clearTraces.Click += new System.EventHandler(this.clearTraces_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 4;
			this.menuItem4.Text = "-";
			// 
			// addFilteredWord
			// 
			this.addFilteredWord.Index = 5;
			this.addFilteredWord.Text = "&Add Filtered Word";
			this.addFilteredWord.Click += new System.EventHandler(this.addFilteredWord_Click);
			// 
			// optionsMenu
			// 
			this.optionsMenu.Index = 6;
			this.optionsMenu.Text = "&Options";
			this.optionsMenu.Click += new System.EventHandler(this.optionsMenu_Click);
			// 
			// visibilityMenu
			// 
			this.visibilityMenu.Index = 7;
			this.visibilityMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																													this.visible100menu,
																													this.visible90menu,
																													this.visible80menu,
																													this.visible70menu,
																													this.visible60menu,
																													this.visible50menu,
																													this.visible40menu,
																													this.visible30menu,
																													this.visible20menu,
																													this.visible10menu});
			this.visibilityMenu.Text = "&Visibility";
			// 
			// visible100menu
			// 
			this.visible100menu.Checked = true;
			this.visible100menu.Index = 0;
			this.visible100menu.Text = "100%";
			this.visible100menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible90menu
			// 
			this.visible90menu.Index = 1;
			this.visible90menu.Text = "90%";
			this.visible90menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible80menu
			// 
			this.visible80menu.Index = 2;
			this.visible80menu.Text = "80%";
			this.visible80menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible70menu
			// 
			this.visible70menu.Index = 3;
			this.visible70menu.Text = "70%";
			this.visible70menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible60menu
			// 
			this.visible60menu.Index = 4;
			this.visible60menu.Text = "60%";
			this.visible60menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible50menu
			// 
			this.visible50menu.Index = 5;
			this.visible50menu.Text = "50%";
			this.visible50menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible40menu
			// 
			this.visible40menu.Index = 6;
			this.visible40menu.Text = "40%";
			this.visible40menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible30menu
			// 
			this.visible30menu.Index = 7;
			this.visible30menu.Text = "30%";
			this.visible30menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible20menu
			// 
			this.visible20menu.Index = 8;
			this.visible20menu.Text = "20%";
			this.visible20menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// visible10menu
			// 
			this.visible10menu.Index = 9;
			this.visible10menu.Text = "10%";
			this.visible10menu.Click += new System.EventHandler(this.changeOpacity);
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 3;
			this.menuItem5.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.filterSelected,
																											 this.monitorProgramMenu,
																											 this.killSelectedProcess});
			this.menuItem5.Text = "&Process";
			// 
			// filterSelected
			// 
			this.filterSelected.Index = 0;
			this.filterSelected.Shortcut = System.Windows.Forms.Shortcut.CtrlF;
			this.filterSelected.Text = "&Filter Selected Program";
			this.filterSelected.Click += new System.EventHandler(this.filterSelected_Click);
			// 
			// monitorProgramMenu
			// 
			this.monitorProgramMenu.Index = 1;
			this.monitorProgramMenu.Shortcut = System.Windows.Forms.Shortcut.CtrlM;
			this.monitorProgramMenu.Text = "&Monitor Selected Program";
			this.monitorProgramMenu.Click += new System.EventHandler(this.monitorProgram_Click);
			// 
			// killSelectedProcess
			// 
			this.killSelectedProcess.Index = 2;
			this.killSelectedProcess.Shortcut = System.Windows.Forms.Shortcut.CtrlK;
			this.killSelectedProcess.Text = "&Kill Selected Process";
			this.killSelectedProcess.Click += new System.EventHandler(this.killProgramMenu_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 4;
			this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																											 this.infoMenu,
																											 this.sendTraceMenu,
																											 this.menuItem9,
																											 this.aboutMenu});
			this.menuItem3.Shortcut = System.Windows.Forms.Shortcut.F1;
			this.menuItem3.Text = "&Help";
			// 
			// infoMenu
			// 
			this.infoMenu.Index = 0;
			this.infoMenu.Shortcut = System.Windows.Forms.Shortcut.F2;
			this.infoMenu.Text = "&Info";
			this.infoMenu.Click += new System.EventHandler(this.infoMenu_Click);
			// 
			// sendTraceMenu
			// 
			this.sendTraceMenu.Index = 1;
			this.sendTraceMenu.Shortcut = System.Windows.Forms.Shortcut.F3;
			this.sendTraceMenu.Text = "&Send Trace";
			this.sendTraceMenu.Click += new System.EventHandler(this.sendTraceMenu_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Index = 2;
			this.menuItem9.Text = "-";
			// 
			// aboutMenu
			// 
			this.aboutMenu.Index = 3;
			this.aboutMenu.Text = "&About";
			this.aboutMenu.Click += new System.EventHandler(this.aboutMenu_Click);
			// 
			// messageList
			// 
			this.messageList.BackColor = System.Drawing.Color.Black;
			this.messageList.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																												  this.procTime,
																												  this.message});
			this.messageList.Dock = System.Windows.Forms.DockStyle.Fill;
			this.messageList.Font = new System.Drawing.Font("Lucida Console", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.messageList.ForeColor = System.Drawing.Color.White;
			this.messageList.FullRowSelect = true;
			this.messageList.Location = new System.Drawing.Point(227, 0);
			this.messageList.Name = "messageList";
			this.messageList.Size = new System.Drawing.Size(653, 478);
			this.messageList.TabIndex = 3;
			this.messageList.View = System.Windows.Forms.View.Details;
			this.messageList.ItemActivate += new System.EventHandler(this.messageList_ItemActivate);
			this.messageList.SelectedIndexChanged += new System.EventHandler(this.messageList_SelectedIndexChanged);
			// 
			// procTime
			// 
			this.procTime.Text = "Proc. Time";
			this.procTime.Width = 85;
			// 
			// message
			// 
			this.message.Text = "Message";
			this.message.Width = 550;
			// 
			// processListPanel
			// 
			this.processListPanel.Controls.Add(this.chevronButton);
			this.processListPanel.Controls.Add(this.processList);
			this.processListPanel.Dock = System.Windows.Forms.DockStyle.Left;
			this.processListPanel.Location = new System.Drawing.Point(0, 0);
			this.processListPanel.Name = "processListPanel";
			this.processListPanel.Size = new System.Drawing.Size(224, 478);
			this.processListPanel.TabIndex = 4;
			// 
			// chevronButton
			// 
			this.chevronButton.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.chevronButton.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
			this.chevronButton.Location = new System.Drawing.Point(0, 455);
			this.chevronButton.Name = "chevronButton";
			this.chevronButton.Size = new System.Drawing.Size(224, 23);
			this.chevronButton.TabIndex = 2;
			this.chevronButton.Text = "<<";
			this.chevronButton.Click += new System.EventHandler(this.chevromButton_Click);
			// 
			// notifyIcon1
			// 
			this.notifyIcon1.ContextMenu = this.contextMenu1;
			this.notifyIcon1.Icon = ((System.Drawing.Icon)(resources.GetObject("notifyIcon1.Icon")));
			this.notifyIcon1.Text = "Trace Monitor";
			this.notifyIcon1.DoubleClick += new System.EventHandler(this.notifyIcon1_DoubleClick);
			// 
			// contextMenu1
			// 
			this.contextMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																												 this.hideRestoreMenu,
																												 this.menuItem11,
																												 this.closeFromIcon});
			// 
			// hideRestoreMenu
			// 
			this.hideRestoreMenu.Index = 0;
			this.hideRestoreMenu.Text = "Hide";
			this.hideRestoreMenu.Click += new System.EventHandler(this.hideRestoreMenu_Click);
			// 
			// menuItem11
			// 
			this.menuItem11.Index = 1;
			this.menuItem11.Text = "-";
			// 
			// closeFromIcon
			// 
			this.closeFromIcon.Index = 2;
			this.closeFromIcon.Text = "Close";
			this.closeFromIcon.Click += new System.EventHandler(this.closeFromIcon_Click);
			// 
			// itemContextMenu
			// 
			this.itemContextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																													 this.filterSelectedProgramContext,
																													 this.monitorSelectedProgramContext,
																													 this.killSelectedProgramContext});
			// 
			// filterSelectedProgramContext
			// 
			this.filterSelectedProgramContext.Index = 0;
			this.filterSelectedProgramContext.Text = "&Filter Selected Program";
			this.filterSelectedProgramContext.Click += new System.EventHandler(this.filterSelected_Click);
			// 
			// monitorSelectedProgramContext
			// 
			this.monitorSelectedProgramContext.Index = 1;
			this.monitorSelectedProgramContext.Text = "&Monitor Selected Program";
			this.monitorSelectedProgramContext.Click += new System.EventHandler(this.monitorProgram_Click);
			// 
			// killSelectedProgramContext
			// 
			this.killSelectedProgramContext.Index = 2;
			this.killSelectedProgramContext.Text = "&Kill Selected Program";
			this.killSelectedProgramContext.Click += new System.EventHandler(this.killProgramMenu_Click);
			// 
			// TraceMonitorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(880, 478);
			this.Controls.Add(this.messageList);
			this.Controls.Add(this.splitter1);
			this.Controls.Add(this.processListPanel);
			this.Menu = this.mainMenu1;
			this.Name = "TraceMonitorForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.Manual;
			this.Text = "TraceMonitor";
			this.Resize += new System.EventHandler(this.TraceMonitorForm_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.TraceMonitorForm_Closing);
			this.Load += new System.EventHandler(this.TraceMonitorForm_Load);
			this.Move += new System.EventHandler(this.TraceMonitorForm_Move);
			this.Activated += new System.EventHandler(this.TraceMonitorForm_Activated);
			this.Deactivate += new System.EventHandler(this.TraceMonitorForm_Deactivate);
			this.processListPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main() {
         Application.Run(new TraceMonitorForm());
      }

      // When the processList item changes, we change the message list
      private void processList_SelectedIndexChanged(object sender, System.EventArgs e) {
         if (processList.SelectedItems.Count > 0) {
            changeDisplayedItem();
         }
      }

      // When the processList item changes, we change the message list
      private void changeDisplayedItem() {
         if (processList.FocusedItem == null || processList.FocusedItem.Tag == null) {
            this.Text = mainTitle;
            if (procNamePanel != null) {
               procNamePanel.CurrentProcessName = "No Process";
               procNamePanel.Update();
            }
         } else {
            WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;

            lock (messageList) {
               messageList.BeginUpdate();
         
               messageList.Items.Clear();
               int count = messageList.Columns.Count;

               if (wpi.ProcessId != 0 && count != 2) {
                  messageList.Columns.RemoveAt(1);
                  messageColumn = 1;
                  resizeColumns(!processListClosed);
               } else if (wpi.ProcessId == 0 && count != 3) {
                  messageList.Columns.Insert(1, "PID", 34, HorizontalAlignment.Left);
                  messageColumn = 2;
                  resizeColumns(!processListClosed);
               }
               activeProcessId = wpi.ProcessId;

               if (activeProcessId == 0) {
                  resizeColumns(!processListClosed);
                  foreach (MessageInfo mi in wpi.Messages) {
                     messageList.Items.Add(
                        new ListViewItem(new String[3]{mi.SecondsSinceStart.ToString(), mi.ProcessId.ToString(), mi.Message}));
                  }
               } else {
                  foreach (MessageInfo mi in wpi.Messages) {
                     messageList.Items.Add(
                        new ListViewItem(new String[2]{mi.SecondsSinceStart.ToString(), mi.Message}));
                  }
               }

               // Focus on the last one in the list when we change processes
               if (messageList.Items.Count > 0) {
                  messageList.Items[messageList.Items.Count-1].EnsureVisible();
               }

               messageList.EndUpdate();
            }
            this.Text = mainTitle + " - " + wpi.DisplayName;
            if (procNamePanel != null) {
               procNamePanel.CurrentProcessName = String.Format("{0} - ({1})", wpi.DisplayName, wpi.ProcessId);
               procNamePanel.Refresh();
            }

         }
      }

      // Exit was clicked, let's close the form.
      private void exitMenu_Click(object sender, System.EventArgs e) {
         // TODO Make this force close
			loggingOff = true;
			this.Close();
		}

		// Check to make sure we can close
		private void TraceMonitorForm_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			// If we don't care about a confirmation, or the confirmation is positive, let's quit!
			if (loggingOff || options.ConfirmQuit == false ||  MessageBox.Show(this, "Do you wish to quit?", "Trace Monitor Confirm Cancel", 
				MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes) {
				dbMon._active = false;
				// After it's set to false, send one last message to cause the event to fire
				Trace.WriteLine("TraceMonitor Ending");

				SzurgotUtilities.SaveProperties.SaveObject(options);
			} else {
				e.Cancel = true;
			}
		}

		protected override void WndProc(ref Message m) {
			if (m.Msg == 0x0011) {
				loggingOff = true;
			}
			base.WndProc (ref m);
		}



      // Helper function for writeConfig.Creates an attribute node for appConfig
      private void addXmlElement(XmlNode node, String name, String key, String value) {
         XmlElement element = node.OwnerDocument.CreateElement(name);

         XmlAttribute attr = node.OwnerDocument.CreateAttribute("key");
         attr.Value = key;
         element.Attributes.Append(attr);

         attr = node.OwnerDocument.CreateAttribute("value");
         attr.Value = value;
         element.Attributes.Append(attr);

         node.AppendChild(element);
      }

      // Checks for delete and removes the appropriate item
      private void processList_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e) {
         if (e.KeyCode == Keys.Delete) {
            // Delete the focused item
            if (processList.FocusedItem != null) {
               lock (this) {
                  ListViewItem lvi = processList.FocusedItem;
                  int index = lvi.Index;
                  WatchedProcessInfo wpi = (WatchedProcessInfo)lvi.Tag;

                  for (int i = 0; i < processList.SelectedItems.Count; i++) {
                     int id = Convert.ToInt32(processList.SelectedItems[i].Text);
                     // Can't delete all traces
                     if (id == 0) continue;
                     wpi.OutputShown = false;
                     wpi.Messages.Clear();
                     
                     // If it's closed program, then we don't need to track anymore
                     if (processList.SelectedItems[i].SubItems[1].Text.StartsWith("*") ) {
                        terminatedCount--;
                        activeDisplays.Remove(id);
                     }

                     // Remove the circular reference
                     processList.SelectedItems[i].Tag = null;

                     processList.SelectedItems[i].Remove();

                     i--;
                  }

                  activeProcessId = -1;

                  if (processList.Items.Count == 0) {
                     messageList.Items.Clear();
                     return;
                  }
                  index--;
                  if (index < 0) {
                     if (processList.Items.Count > 0) {
                        processList.Items[0].Focused = true;
                     }
                  } else if (index > processList.Items.Count) {
                     processList.Items[processList.Items.Count - 1].Focused = true;
                  } else {
                     processList.Items[index].Focused = true;
                  }
                  if (processList.FocusedItem != null) {
                     processList.FocusedItem.Selected = true;
                  } else {
                     messageList.Items.Clear();
                     this.Text = mainTitle;
                  }
               }
            }
         }
      }

      // Menu item for setting or removing the monitored program.
      private void monitorProgram_Click(Object sender, EventArgs e) {
         if (options.MonitoredProgram == null || options.MonitoredProgram == "") {
            if (processList.SelectedItems.Count == 1) {
               WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;
               options.MonitoredProgram = wpi.DisplayName;

               monitorProgramMenu.Text = "Stop Monitoring - " + options.MonitoredProgram;
            }
         } else {
            options.MonitoredProgram = "";
            monitorProgramMenu.Text = "Monitor Selected Program";
         }
      }


      // Menu item to kill the selected program.
      private void killProgramMenu_Click(Object sender, EventArgs e) {
         if (processList.SelectedItems.Count == 1) {
            WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;

            try {
               Process p = Process.GetProcessById(wpi.ProcessId);
               p.Kill();
            } catch (Exception ex) {
               MessageBox.Show("Shutdown Excption\n" + ex);
            }
         }
      }

      // Menu item to set the current item to no longer display output
      private void filterSelected_Click(object sender, System.EventArgs e) {
         if (processList.FocusedItem != null) {
            WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;
            lock (this) {
               if (!options.FilteredPrograms.Contains(wpi.DisplayName) ) {
                  options.FilteredPrograms.Add(wpi.DisplayName);
               }

               if (wpi.ProcessId== activeProcessId) {
                  activeProcessId = -1;
               }

               ArrayList removedItems = new ArrayList();
               for (int i = 0; i < processList.Items.Count && processList.Items.Count > 0; i++) {
                  WatchedProcessInfo scannedWpi= (WatchedProcessInfo)processList.Items[i].Tag;
                  if (scannedWpi.DisplayName == wpi.DisplayName) {
                     removedItems.Add(processList.Items[i]);
                     if (activeDisplays.Contains(scannedWpi.ProcessId) ) {
                        activeDisplays.Remove(scannedWpi.ProcessId);
                     }
                  }
               }
               foreach (ListViewItem lvi in removedItems) {
                  lvi.Remove();
               }
               messageList.Items.Clear();
            }

            if (processList.FocusedItem == null) {
               processList.Items[0].Selected = true;
//               if (procNamePanel != null) {
//                  procNamePanel.CurrentProcessName = "No Process Selected";
//                  procNamePanel.Refresh();
//               }
               this.Text = mainTitle;
            } else {
               processList.FocusedItem.Selected = true;
               
            }
         }
      }

      // Helper function which displays a color dialog and returns the selected color.
      private Color getColor(Color c) {
         ColorDialog cd = new ColorDialog();
         cd.Color = c;
         if (cd.ShowDialog(this) == DialogResult.OK) {
            c = cd.Color;
         }
         return c;
      }

      // Final form load. Sets the size and position. and starts the monitor thread.
      private void TraceMonitorForm_Load(object sender, System.EventArgs e) {
         // Create the all traces object, and add it at the top of the list
         ListViewItem lvi = new ListViewItem(new String[2]{"0", allTraces.DisplayName});
         lvi.Tag = allTraces;
         allTraces.ListItem = lvi; // Circular Reference

         // Add it so that it's at the top of the list and focused
         processList.Items.Insert(0, lvi);
         lvi.Focused = true;
         lvi.Selected = true;
         
         
         monitorThread = new Thread(new ThreadStart(MonitorThreadProcess));
         monitorThread.IsBackground = true;
         monitorThread.Start();

         // Set the Size & location here because it doesn't set properly in the Constructor
         if (options.FormBounds != Rectangle.Empty) {
            // When used in a multimonitor setup, sometimes it thinks it's *way* off the screen.
            // If the form won't display on the visible area, reset it to the 0,0 position, which is the primary monitor

            Rectangle r = SystemInformation.VirtualScreen;

            if (options.FormBounds.IntersectsWith(r) ) {
               this.Bounds = options.FormBounds;
            } else {
               this.Bounds = new Rectangle(0, 0, 640, 480);
            }
         }
      }

      // Sends a basic trace back to itself
      private void sendTraceMenu_Click(object sender, System.EventArgs e) {
         Trace.WriteLine("Trace Message to self @ " + DateTime.Now);
      }

      // Displays the options dialog (a property grid)
      private void optionsMenu_Click(object sender, System.EventArgs e) {
         OptionsForm of = new OptionsForm(options);
         of.propertyValueChanged += new EventHandler(onPropertyValueChanged);
         of.ShowDialog(this);
      }

      // When a property changes, set it in real time.
      private void onPropertyValueChanged(Object sender, EventArgs e) {
         setOptions();
      }

      // sends an debug message back to itself
      private void infoMenu_Click(object sender, System.EventArgs e) {
         Process p = Process.GetCurrentProcess();
         Trace.WriteLine("Basic Info:" + p.MainModule.ModuleName + ":\r\n" + p.WorkingSet + ":" + this.Bounds);
      }

      // Show the filtered program list.
      private void showFilteredProgramList_Click(object sender, System.EventArgs e) {
         FilteredList fl = new FilteredList(options.FilteredPrograms);
         fl.ShowDialog(this);
      }

      Rectangle oldBounds = Rectangle.Empty;
      ProcessNamePanel procNamePanel = null;
      
      // Open or close the process list panel
      private void chevromButton_Click(object sender, System.EventArgs e) {
         if (chevronButton.Text == "<<") {
            processListClosed = true;
         
            oldBounds = processListPanel.Bounds;
            processListPanel.SetBounds(oldBounds.X, oldBounds.X, 30, oldBounds.Height);

            processListPanel.Controls.Remove(processList);

            String procName = "No Process Selected";
            if (processList.FocusedItem != null) {
               WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;
               procName = String.Format("{0} - ({1})", wpi.DisplayName, wpi.ProcessId);

            }
         
            procNamePanel = new ProcessNamePanel(procName);
            procNamePanel.Dock = DockStyle.Fill;
            procNamePanel.Click += new EventHandler(processPanel_Click);
            processListPanel.Controls.Add(procNamePanel);

            splitter1.Enabled = false;

            chevronButton.Text = ">>";
            resizeColumns(false);
         } else {
            processListClosed = true;
            openPanel();
            resizeColumns(true);
         }
      }

      // Open the process list panel
      private void processPanel_Click(Object sender, EventArgs e) {
         openPanel();
      }

      // Open the process list panel
      private void openPanel() {
         processListPanel.Controls.Remove(procNamePanel);
         processListPanel.Controls.Add(processList);

         processListPanel.SetBounds(oldBounds.X, oldBounds.Y, oldBounds.Width, oldBounds.Height);

         procNamePanel = null;

         splitter1.Enabled = true;

         chevronButton.Text = "<<";
      }

      // From IViewableList
      public bool getPreviousMessage(out String messageTime, out String message) {
         return getMessage(-1, out messageTime, out message);
      }

      // From IViewableList
      public bool getNextMessage(out String messageTime, out String message) {
         return getMessage(1, out messageTime, out message);
      }

      // Actually gets the message for getNext & getPrevious
      private bool getMessage(int direction, out String messageTime, out String message) {
         messageTime = "";
         message = "";
         if (direction == -1 && messageList.FocusedItem.Index == 0) {
            MessageBox.Show("No Previous Message");
            return false;
         } else if (direction == 1 && messageList.FocusedItem.Index >= messageList.Items.Count - 1) {
            MessageBox.Show("No Next Message");
            return false;
         }

         messageList.Items[messageList.FocusedItem.Index + direction].Focused = true;
         messageTime = messageList.FocusedItem.Text;
         message = messageList.FocusedItem.SubItems[messageColumn].Text;
         return true;
      }

      // When an individual message is clicked,  this will display a detail form for that message.
      private void messageList_ItemActivate(object sender, System.EventArgs e) {
         ListViewItem process = processList.FocusedItem;
         WatchedProcessInfo wpi = (WatchedProcessInfo)process.Tag;
         MessageDetail md = new MessageDetail(wpi.ProcessId, wpi.DisplayName, messageList.FocusedItem.Text, 
            messageList.FocusedItem.SubItems[messageColumn].Text, this);
         md.ShowDialog(this);
      }

      // Setting to allow the program to minimize to tray
      private void minimizeToTray_Click(object sender, System.EventArgs e) {
         minimizeToTray.Checked = !minimizeToTray.Checked;
         notifyIcon1.Visible = minimizeToTray.Checked;
         options.MinimizeToTray = minimizeToTray.Checked;
         if (minimizeToTray.Checked)
            notifyIcon1.Icon = standardIcon;
      }

      // Capture the resize event to see if it's been minimized
      private void TraceMonitorForm_Resize(object sender, System.EventArgs e) {
         if (minimizeToTray.Checked && this.WindowState == FormWindowState.Minimized) {
            this.Visible = false;
				hideRestoreMenu.Text = "Restore";
         } else {
				hideRestoreMenu.Text = "Hide";
            resizeColumns(!processListClosed);
				if (this.Visible) options.FormBounds = this.Bounds;
         }
      }

      private void notifyIcon1_DoubleClick(object sender, System.EventArgs e) {
			if (this.WindowState == FormWindowState.Minimized) {
				this.Visible = true;
				this.WindowState = FormWindowState.Normal;
				notifyIcon1.Icon = standardIcon;
				flashingIcon = false;
			} else {
				// this.BringToFront();
            SetForegroundWindow(this.Handle);
			}
      }

      // Shows the filtered word list to allow the user to view/remove filtered words.
      private void showFillteredWordList_Click(object sender, System.EventArgs e) {
         FilteredList fl = new FilteredList(options.FilteredWords);
         fl.ShowDialog(this);
      }

      // Shows the AddFilteredWord modal dialog, and adds the selected word.
      private void addFilteredWord_Click(object sender, System.EventArgs e) {
         AddFilteredWordForm afw = new AddFilteredWordForm();
         if (afw.ShowDialog(this) == DialogResult.OK) {
            options.FilteredWords.Add(afw.FilteredWord);
         }
      }

      // Capture the splitter move to properly resize the form.
      private void splitter1_SplitterMoved(object sender, System.Windows.Forms.SplitterEventArgs e) {
         resizeColumns(!processListClosed);
			options.SplitterPosition = splitter1.SplitPosition;
      }

      // This attempts to resize the form so that there is a vertical scrollbar, but no horizontal scrollbar by
      // factoring in the widths of the columns, and spacing between them.
      private void resizeColumns(bool resizeProcessList) {
         // When resizing, factor in the spacing between the columns, and the width of potential scrollbars
         if (resizeProcessList) {
            processList.Columns[0].Width = 34;
            processList.Columns[1].Width = processList.Size.Width - (34 + 5 + 16);
         }

         if (messageList.Columns.Count == 3) {
            messageList.Columns[0].Width = 85;
            messageList.Columns[1].Width = 40;
            messageList.Columns[2].Width = messageList.Size.Width - (85 + 5 + 40 + 16);
         } else if (messageList.Columns.Count == 2) {
            messageList.Columns[0].Width = 85;
            messageList.Columns[1].Width = messageList.Size.Width - (85 + 5 + 16);
         } else {
            MessageBox.Show("Not enough columns!");
         }
      }

      // Menu item to clear the "Current Traces" List.
      private void clearTraces_Click(object sender, System.EventArgs e) {
         lock (messageList) {
#if ONLYCLEARALL
				if (activeProcessId == 0) {
				
					// TODO Make sure that the all traces are showing
					allTraces.Messages.Clear();
					if (activeProcessId == 0) {
						messageList.Items.Clear();
					}
				} else {
					MessageBox.Show("Can only clear 'All Traces' while viewing 'All Traces' list");
				}
#else
				WatchedProcessInfo wpi = (WatchedProcessInfo)processList.FocusedItem.Tag;

				wpi.Messages.Clear();
				messageList.Items.Clear();
#endif
         }
      }

      // Copy the text of the selected message to the clipboard
      private void copyMenu_Click(object sender, System.EventArgs e) {
         StringBuilder sb = new StringBuilder();
         int offset = 1;
         if (messageList.Columns.Count == 3)
            offset = 2;
         if (messageList.SelectedItems.Count != 0) {
            foreach (ListViewItem lvi in messageList.SelectedItems) {
               sb.Append(lvi.SubItems[offset].Text);
               sb.Append("\r\n");
            }
            Clipboard.SetDataObject(sb.ToString(), true);
         }
      }

      // Select all the items in the displayed message list
      private void selectAllMenu_Click(object sender, System.EventArgs e) {
         for (int i = 0; i < messageList.Items.Count; i++) {
            messageList.Items[i].Selected = true;
         }
      }

      private void saveTraceMenu_Click(object sender, System.EventArgs e) {
         StringBuilder sb = new StringBuilder();
         int offset = 1;
         if (messageList.Columns.Count == 3)
            offset = 2;
         if (messageList.Items.Count != 0) {
            foreach (ListViewItem lvi in messageList.Items) {
               sb.Append(lvi.SubItems[offset].Text);
               sb.Append("\r\n");
            }
            String trace = sb.ToString();

            SaveFileDialog sfd = new SaveFileDialog();
            sfd.AddExtension = true;
            sfd.DefaultExt = "txt";
            sfd.OverwritePrompt = true;
            sfd.ValidateNames = true;
            sfd.FileName = "Trace.txt";
            if (sfd.ShowDialog(this) == DialogResult.OK) {
               MessageBox.Show("FileName:" + sfd.FileName);
               StreamWriter sw = new StreamWriter(sfd.FileName, false);
               sw.Write(trace);
               sw.Close();
            }

         } else {
            MessageBox.Show("No data to save");
         }
      }

      private void closeFromIcon_Click(object sender, System.EventArgs e) {
         this.Close();
      }

		private void TraceMonitorForm_Move(object sender, System.EventArgs e) {
			if (this.WindowState == FormWindowState.Normal) {
				options.FormBounds = this.Bounds;
			}
		}

		private void hideRestoreMenu_Click(object sender, System.EventArgs e) {
			if (hideRestoreMenu.Text == "Hide") {
				this.WindowState = FormWindowState.Minimized;
			} else {
				// Call the event so no steps are missed
				notifyIcon1_DoubleClick(this, EventArgs.Empty);
			}
		}

		private void aboutMenu_Click(object sender, System.EventArgs e) {
			MessageBox.Show("Trace Monitor");
		}

		private void TraceMonitorForm_Activated(object sender, System.EventArgs e) {
			activated = true;
			this.flashingIcon = false;
		}

		private void TraceMonitorForm_Deactivate(object sender, System.EventArgs e) {
			activated = false;
		}

      private void processList_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e) {
         if (e.Button == MouseButtons.Right) {
            ListViewItem lvi = processList.GetItemAt(e.X, e.Y);
            if (lvi != null && lvi.Text != "0") {
               itemContextMenu.Show(processList, new Point(e.X, e.Y));
            }
         }
      }

		private void changeOpacity(object sender, System.EventArgs e) {
			MenuItem mi = (MenuItem)sender;
			Trace.WriteLine("Menu Item:" + mi.Text);
			if (mi.Text == "100%") {
				this.Opacity = 100;
			} else {
				int pct = Convert.ToInt32(mi.Text.Substring(0,2));
				this.Opacity = ((float)pct / (float)100);
			}
			options.Opacity = this.Opacity;

			foreach (MenuItem child in visibilityMenu.MenuItems) {
				child.Checked = (child.Text == mi.Text);
			}
			foreach (MenuItem child in contextVisibilityMenu.MenuItems) {
				child.Checked = (child.Text == mi.Text);
			}
		}

		private void messageList_SelectedIndexChanged(object sender, System.EventArgs e) {
		
		}

      // Delegate for the marshalling of the event back to the main thread
      private delegate void TraceMonitorMessageDelegate();

      [System.Runtime.InteropServices.DllImport("User32.dll")]
      private static extern bool SetForegroundWindow(IntPtr hwnd);
   }
}
