using System;
using System.Collections;
using System.Text;
using System.Windows.Forms;

namespace TraceMonitor {

   /// <summary>
   /// Information on an individual process as well as message list functions.
   /// </summary>
   public class WatchedProcessInfo {
      ArrayList _messages;
      /// <summary>
      /// The current list of message received from the trace output.
      /// </summary>
      public ArrayList Messages{
         get { return _messages; }
      }

      private String _displayName;
      /// <summary>
      /// The display name of the process. Defaults to the process id if the name cannot be retrieved.
      /// </summary>
      public String DisplayName {
         get { return _displayName; }
      }

      /// <summary>
      /// Append the message with the default ProcessID &amp; Start Time
      /// </summary>
      /// <param name="text">Text of the message</param>
      /// <returns>The MessageInfo object</returns>
      public MessageInfo AppendMessage(String text) {
         MessageInfo mi = new MessageInfo(_processId,  text, _processStartTime);
         _messages.Add(mi);
         return mi;
      }

      /// <summary>
      /// Append the message with explicit ProcessId &amp; StartTime (used for aggregate list)
      /// </summary>
      /// <param name="text">Text of the message.</param>
      /// <param name="ProcessId">ProcessId the message was sent from.</param>
      /// <param name="ProcessStartTime">StartTime of the process the message was sent from.</param>
      /// <returns></returns>
      public MessageInfo AppendMessage(String text, int ProcessId, DateTime ProcessStartTime) {
         MessageInfo mi = new MessageInfo(ProcessId, text, ProcessStartTime);
         _messages.Add(mi);
         return mi;
      }

      private DateTime _processStartTime;
      /// <summary>
      /// Start time of this process.
      /// </summary>
      public DateTime ProcessStartTime {
         get { return _processStartTime; }
      }

      private int _processId;
      /// <summary>
      /// ProcessId of this process.
      /// </summary>
      public int ProcessId {
         get { return _processId; }
      }

      private bool _filtered;
      /// <summary>
      /// Has the program been filtered out.
      /// </summary>
      public bool Filtered {
         get { return _filtered; }
         set { _filtered = value; }
      }

      private ListViewItem _listItem;
      /// <summary>
      /// The list item of the process. (Creates a circular reference, since this object is the ListItems tag object.
      /// </summary>
      public ListViewItem ListItem {
         get { return _listItem; }
         set { _listItem = value; }
      }

      private bool _outputShown = false;
      /// <summary>
      /// Is the output currently being shown from this Process.
      /// </summary>
      public bool OutputShown {
         get { return _outputShown; }
         set { _outputShown = value; }
      }
      
      /// <summary>
      /// Constructor.
      /// </summary>
      /// <param name="processId">Process Id.</param>
      /// <param name="displayName">Display Name of the Process.</param>
      /// <param name="processStartTime">Start Time of the process.</param>
      public WatchedProcessInfo(int processId, String displayName, DateTime processStartTime) {
         _messages = new ArrayList();
         _processId = processId;

         _processStartTime = processStartTime;

         _displayName = displayName;
      }
   }
}
